@@ -132,6 +132,116 @@ class Horde_Date
      */
     protected $_timezone;
 
+    /**
+     * These aliases map Windows and Lotus Timezone IDs to those understood by PHP
+     * This list better moves somewhere else
+     * @var array
+     */
+    protected static $_timezoneAliases = array(
+        'Dateline Standard Time' => 'Etc/GMT+12',
+        'UTC-11' => 'Etc/GMT+11',
+        'Hawaiian Standard Time' => 'Pacific/Honolulu',
+        'Alaskan Standard Time' => 'America/Anchorage',
+        'Pacific Standard Time (Mexico)' => 'America/Santa_Isabel',
+        'Pacific Standard Time' => 'America/Los_Angeles',
+        'US Mountain Standard Time' => 'America/Phoenix',
+        'Mountain Standard Time (Mexico)' => 'America/Chihuahua',
+        'Mountain Standard Time' => 'America/Denver',
+        'Central America Standard Time' => 'America/Guatemala',
+        'Central Standard Time' => 'America/Chicago',
+        'Central Standard Time (Mexico)' => 'America/Mexico_City',
+        'Canada Central Standard Time' => 'America/Regina',
+        'SA Pacific Standard Time' => 'America/Bogota',
+        'Eastern Standard Time' => 'America/New_York',
+        'US Eastern Standard Time' => 'America/Indianapolis',
+        'Venezuela Standard Time' => 'America/Caracas',
+        'Paraguay Standard Time' => 'America/Asuncion',
+        'Atlantic Standard Time' => 'America/Halifax',
+        'Central Brazilian Standard Time' => 'America/Cuiaba',
+        'SA Western Standard Time' => 'America/La_Paz',
+        'Pacific SA Standard Time' => 'America/Santiago',
+        'Newfoundland Standard Time' => 'America/St_Johns',
+        'E. South America Standard Time' => 'America/Sao_Paulo',
+        'Argentina Standard Time' => 'America/Buenos_Aires',
+        'SA Eastern Standard Time' => 'America/Cayenne',
+        'Greenland Standard Time' => 'America/Godthab',
+        'Montevideo Standard Time' => 'America/Montevideo',
+        'Bahia Standard Time' => 'America/Bahia',
+        'UTC-02' => 'Etc/GMT+2',
+        'Azores Standard Time' => 'Atlantic/Azores',
+        'Cape Verde Standard Time' => 'Atlantic/Cape_Verde',
+        'Morocco Standard Time' => 'Africa/Casablanca',
+        'GMT Standard Time' => 'Europe/London',
+        'Greenwich Standard Time' => 'Atlantic/Reykjavik',
+        'W. Europe Standard Time' => 'Europe/Berlin',
+        'Central Europe Standard Time' => 'Europe/Budapest',
+        'Romance Standard Time' => 'Europe/Paris',
+        'Central European Standard Time' => 'Europe/Warsaw',
+        'W. Central Africa Standard Time' => 'Africa/Lagos',
+        'Namibia Standard Time' => 'Africa/Windhoek',
+        'Jordan Standard Time' => 'Asia/Amman',
+        'GTB Standard Time' => 'Europe/Bucharest',
+        'Middle East Standard Time' => 'Asia/Beirut',
+        'Egypt Standard Time' => 'Africa/Cairo',
+        'Syria Standard Time' => 'Asia/Damascus',
+        'E. Europe Standard Time' => 'Asia/Nicosia',
+        'South Africa Standard Time' => 'Africa/Johannesburg',
+        'FLE Standard Time' => 'Europe/Kiev',
+        'Turkey Standard Time' => 'Europe/Istanbul',
+        'Israel Standard Time' => 'Asia/Jerusalem',
+        'Arabic Standard Time' => 'Asia/Baghdad',
+        'Kaliningrad Standard Time' => 'Europe/Kaliningrad',
+        'Arab Standard Time' => 'Asia/Riyadh',
+        'E. Africa Standard Time' => 'Africa/Nairobi',
+        'Iran Standard Time' => 'Asia/Tehran',
+        'Arabian Standard Time' => 'Asia/Dubai',
+        'Azerbaijan Standard Time' => 'Asia/Baku',
+        'Russian Standard Time' => 'Europe/Moscow',
+        'Mauritius Standard Time' => 'Indian/Mauritius',
+        'Georgian Standard Time' => 'Asia/Tbilisi',
+        'Caucasus Standard Time' => 'Asia/Yerevan',
+        'Afghanistan Standard Time' => 'Asia/Kabul',
+        'Pakistan Standard Time' => 'Asia/Karachi',
+        'West Asia Standard Time' => 'Asia/Tashkent',
+        'India Standard Time' => 'Asia/Calcutta',
+        'Sri Lanka Standard Time' => 'Asia/Colombo',
+        'Nepal Standard Time' => 'Asia/Katmandu',
+        'Central Asia Standard Time' => 'Asia/Almaty',
+        'Bangladesh Standard Time' => 'Asia/Dhaka',
+        'Ekaterinburg Standard Time' => 'Asia/Yekaterinburg',
+        'Myanmar Standard Time' => 'Asia/Rangoon',
+        'SE Asia Standard Time' => 'Asia/Bangkok',
+        'N. Central Asia Standard Time' => 'Asia/Novosibirsk',
+        'China Standard Time' => 'Asia/Shanghai',
+        'North Asia Standard Time' => 'Asia/Krasnoyarsk',
+        'Singapore Standard Time' => 'Asia/Singapore',
+        'W. Australia Standard Time' => 'Australia/Perth',
+        'Taipei Standard Time' => 'Asia/Taipei',
+        'Ulaanbaatar Standard Time' => 'Asia/Ulaanbaatar',
+        'North Asia East Standard Time' => 'Asia/Irkutsk',
+        'Tokyo Standard Time' => 'Asia/Tokyo',
+        'Korea Standard Time' => 'Asia/Seoul',
+        'Cen. Australia Standard Time' => 'Australia/Adelaide',
+        'AUS Central Standard Time' => 'Australia/Darwin',
+        'E. Australia Standard Time' => 'Australia/Brisbane',
+        'AUS Eastern Standard Time' => 'Australia/Sydney',
+        'West Pacific Standard Time' => 'Pacific/Port_Moresby',
+        'Tasmania Standard Time' => 'Australia/Hobart',
+        'Yakutsk Standard Time' => 'Asia/Yakutsk',
+        'Central Pacific Standard Time' => 'Pacific/Guadalcanal',
+        'Vladivostok Standard Time' => 'Asia/Vladivostok',
+        'New Zealand Standard Time' => 'Pacific/Auckland',
+        'UTC+12' => 'Etc/GMT-12',
+        'Fiji Standard Time' => 'Pacific/Fiji',
+        'Magadan Standard Time' => 'Asia/Magadan',
+        'Tonga Standard Time' => 'Pacific/Tongatapu',
+        'Samoa Standard Time' => 'Pacific/Apia',
+        // Found from Lotus Notes
+        'W. Europe' => 'Europe/Berlin',
+        'E. Europe' => 'Asia/Nicosia',
+    );
+
+
     /**
      * Default format for __toString()
      *
@@ -235,6 +345,10 @@ class Horde_Date
             $this->_hour = $this->_min = $this->_sec = 0;
         } elseif ((string)(int)$date == $date) {
             // Try as a timestamp.
+            if ($this->_timezone) {
+                $oldtimezone = date_default_timezone_get();
+                date_default_timezone_set($this->_timezone);
+            }
             $parts = @getdate($date);
             if ($parts) {
                 $this->_year  = $parts['year'];
@@ -244,20 +358,31 @@ class Horde_Date
                 $this->_min   = $parts['minutes'];
                 $this->_sec   = $parts['seconds'];
             }
+            if ($this->_timezone) {
+                date_default_timezone_set($oldtimezone);
+            }
         } else {
-            try {
-                $parsed = new DateTime($date);
-            } catch (Exception $e) {
-                throw new Horde_Date_Exception(sprintf(Horde_Date_Translation::t("Failed to parse time string (%s)"), $date));
+            if (!empty($timezone)) {
+                try {
+                    $parsed = new DateTime($date, new DateTimeZone($timezone));
+                } catch (Exception $e) {
+                    throw new Horde_Date_Exception(sprintf(Horde_Date_Translation::t("Failed to parse time string (%s)"), $date));
+                }
+            } else {
+                try {
+                    $parsed = new DateTime($date);
+                } catch (Exception $e) {
+                    throw new Horde_Date_Exception(sprintf(Horde_Date_Translation::t("Failed to parse time string (%s)"), $date));
+                }
+                $parsed->setTimezone(new DateTimeZone(date_default_timezone_get()));
+                $this->_initializeTimezone(date_default_timezone_get());
             }
-            $parsed->setTimezone(new DateTimeZone(date_default_timezone_get()));
             $this->_year  = (int)$parsed->format('Y');
             $this->_month = (int)$parsed->format('m');
             $this->_mday  = (int)$parsed->format('d');
             $this->_hour  = (int)$parsed->format('H');
             $this->_min   = (int)$parsed->format('i');
             $this->_sec   = (int)$parsed->format('s');
-            $this->_initializeTimezone(date_default_timezone_get());
         }
     }
 
@@ -529,6 +654,10 @@ class Horde_Date
     public function setTimezone($timezone)
     {
         $date = $this->toDateTime();
+        /* Workaround for standard cases of bug #11688 */
+        if (array_key_exists($timezone, self::$_timezoneAliases)) {
+            $timezone = self::$_timezoneAliases[$timezone];
+        }
         $date->setTimezone(new DateTimeZone($timezone));
         $this->_timezone = $timezone;
         $this->_year     = (int)$date->format('Y');
@@ -1000,7 +1129,9 @@ class Horde_Date
             $this->_correctMonth();
             /* When correcting the month, always correct the day too. Months
              * have different numbers of days. */
-            $mask |= self::MASK_DAY;
+            if (isset($this->_mday)) {
+                $mask |= self::MASK_DAY;
+            }
         }
 
         if ($mask & self::MASK_DAY) {
@@ -1125,8 +1256,11 @@ class Horde_Date
     {
         if (empty($timezone)) {
             $timezone = date_default_timezone_get();
+        } elseif (array_key_exists($timezone, self::$_timezoneAliases)) {
+            /* Workaround for standard cases of bug #11688 */
+            $timezone = self::$_timezoneAliases[$timezone];
         }
+
         $this->_timezone = $timezone;
     }
-
 }
