@@ -2,7 +2,7 @@
 /**
  * This file contains the Horde_Date_Recurrence class and according constants.
  *
- * Copyright 2007-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2007-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -1123,13 +1123,23 @@ class Horde_Date_Recurrence
                 break;
             }
 
+            // MUST take into account the time portion if it is present.
+            // See Bug: 12869 and Bug: 2813
             if (isset($rdata['UNTIL'])) {
-                list($year, $month, $mday) = sscanf($rdata['UNTIL'],
-                                                    '%04d%02d%02d');
-                $this->setRecurEnd(new Horde_Date(array('year' => $year,
-                                                        'month' => $month,
-                                                        'mday' => $mday),
-                                                  $this->start->timezone));
+                if (preg_match('/^(\d{4})-?(\d{2})-?(\d{2})T? ?(\d{2}):?(\d{2}):?(\d{2})(?:\.\d+)?(Z?)$/', $rdata['UNTIL'], $parts)) {
+                    $until = new Horde_Date($rdata['UNTIL'], 'UTC');
+                    $until->setTimezone($this->start->timezone);
+                } else {
+                    list($year, $month, $mday) = sscanf($rdata['UNTIL'],
+                                                        '%04d%02d%02d');
+                    $until = new Horde_Date(
+                        array('year' => $year,
+                              'month' => $month,
+                              'mday' => $mday + 1),
+                        $this->start->timezone
+                    );
+                }
+                $this->setRecurEnd($until);
             }
             if (isset($rdata['COUNT'])) {
                 $this->setRecurCount($rdata['COUNT']);
@@ -1601,4 +1611,68 @@ class Horde_Date_Recurrence
         return $json;
     }
 
+    /**
+     * Output a human readable description of the recurrence rule.
+     *
+     * @return string
+     * @since 2.1.0
+     */
+    public function toString($date_format)
+    {
+        $string = '';
+        if ($this->hasRecurType(self::RECUR_DAILY)) {
+                $string = _("Daily: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("day(s)");
+        } elseif ($this->hasRecurType(self::RECUR_WEEKLY)) {
+            $weekdays = array();
+            if ($this->recurOnDay(Horde_Date::MASK_MONDAY))    $weekdays[] = _("Monday");
+            if ($this->recurOnDay(Horde_Date::MASK_TUESDAY))   $weekdays[] = _("Tuesday");
+            if ($this->recurOnDay(Horde_Date::MASK_WEDNESDAY)) $weekdays[] = _("Wednesday");
+            if ($this->recurOnDay(Horde_Date::MASK_THURSDAY))  $weekdays[] = _("Thursday");
+            if ($this->recurOnDay(Horde_Date::MASK_FRIDAY))    $weekdays[] = _("Friday");
+            if ($this->recurOnDay(Horde_Date::MASK_SATURDAY))  $weekdays[] = _("Saturday");
+            if ($this->recurOnDay(Horde_Date::MASK_SUNDAY))    $weekdays[] = _("Sunday");
+            $string = _("Weekly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("week(s) on:") . ' ' . implode(', ', $weekdays);
+        } elseif ($this->hasRecurType(self::RECUR_MONTHLY_DATE)) {
+            $string = _("Monthly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("month(s)") . ' ' . _("on the same date");
+        } elseif ($this->hasRecurType(self::RECUR_MONTHLY_WEEKDAY)) {
+            $string = _("Monthly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("month(s)") . ' ' . _("on the same weekday");
+        } elseif ($this->hasRecurType(self::RECUR_YEARLY_DATE)) {
+            $string =  _("Yearly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("year(s) on the same date");
+        } elseif ($this->hasRecurType(self::RECUR_YEARLY_DAY)) {
+           $string = _("Yearly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("year(s) on the same day of the year");
+        } elseif ($this->hasRecurType(self::RECUR_YEARLY_WEEKDAY)) {
+           $string = _("Yearly: Recurs every") . ' ' . $this->getRecurInterval() . ' ' . _("year(s) on the same weekday and month of the year");
+        }
+
+        $string .= "\n" . _("Ends after") . ': ' . ($this->hasRecurEnd() ? $this->recurEnd->strftime($date_format) . ($this->recurEnd->hour == 23 && $this->recurEnd->min == 59 ? '' : ' ' . $this->recurEnd->format($date_format)) : ($this->getRecurCount() ? sprintf(_("%d times"), $this->getRecurCount()) : _("No end date")));;
+        if ($this->getExceptions()) {
+            $string .= "\n" . _("Exceptions on") . ': ';
+            foreach ($this->getExceptions() as $exception_date) {
+                $string .= $this->_formatExceptionDate($exception_date, $date_format) . ' ';
+            }
+        }
+
+        return $string;
+    }
+
+    /**
+     * Returns a correcty formatted exception date for recurring events.
+     *
+     * @param string $date    Exception in the format Ymd.
+     * @param string $format  The format to display in.
+     *
+     * @return string  The formatted date and delete link.
+     * @since 2.1.0
+     */
+    protected function _formatExceptionDate($date, $format)
+    {
+        if (!preg_match('/(\d{4})(\d{2})(\d{2})/', $date, $match)) {
+            return '';
+        }
+        $horde_date = new Horde_Date(array('year' => $match[1],
+                                           'month' => $match[2],
+                                           'mday' => $match[3]));
+        return $horde_date->strftime($format);
+    }
+
 }
