@@ -531,7 +531,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(2, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-08 09:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-08 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('D2 #4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_DAILY, $r->getRecurType());
@@ -544,14 +544,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(2, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-07 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-08 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=DAILY;INTERVAL=2;UNTIL=20070308T090000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_DAILY, $r->getRecurType());
         $this->assertEquals(2, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-08 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-08 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=DAILY;INTERVAL=2;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_DAILY, $r->getRecurType());
@@ -575,7 +575,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertEquals(Horde_Date::MASK_THURSDAY, $r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-30 08:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-30 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('W1 SU MO TU WE TH FR SA 20070603T235959');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_WEEKLY, $r->getRecurType());
@@ -601,14 +601,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertEquals(Horde_Date::MASK_THURSDAY, $r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-29 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-30 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=WEEKLY;INTERVAL=1;BYDAY=TH;UNTIL=20070330T080000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_WEEKLY, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertEquals(Horde_Date::MASK_THURSDAY, $r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-03-30 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-03-30 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=WEEKLY;INTERVAL=1;BYDAY=TH;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_WEEKLY, $r->getRecurType());
@@ -650,7 +650,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('MD1 1 #4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_DATE, $r->getRecurType());
@@ -669,14 +669,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-01 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=MONTHLY;INTERVAL=1;UNTIL=20070502T080000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_DATE, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-02 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=MONTHLY;INTERVAL=1;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_DATE, $r->getRecurType());
@@ -706,7 +706,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('MP1 1+ TH #4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_WEEKDAY, $r->getRecurType());
@@ -719,14 +719,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-01 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=MONTHLY;INTERVAL=1;BYDAY=1TH;UNTIL=20070502T080000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_WEEKDAY, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2007-05-02 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2007-05-02 08:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=MONTHLY;INTERVAL=1;BYDAY=1TH;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_MONTHLY_WEEKDAY, $r->getRecurType());
@@ -750,7 +750,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('YM1 3 #4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DATE, $r->getRecurType());
@@ -763,14 +763,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-01 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;UNTIL=20090302T090000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DATE, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DATE, $r->getRecurType());
@@ -794,7 +794,7 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule10('YD1 60 #4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DAY, $r->getRecurType());
@@ -807,14 +807,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-01 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;BYYEARDAY=60;UNTIL=20090302T090000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DAY, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;BYYEARDAY=60;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_DAY, $r->getRecurType());
@@ -831,14 +831,14 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-01 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;BYDAY=1TH;BYMONTH=3;UNTIL=20090302T090000Z');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_WEEKDAY, $r->getRecurType());
         $this->assertEquals(1, $r->getRecurInterval());
         $this->assertNull($r->getRecurOnDays());
         $this->assertNull($r->getRecurCount());
-        $this->assertEquals('2009-03-02 00:00:00', (string)$r->recurEnd);
+        $this->assertEquals('2009-03-02 09:00:00', (string)$r->recurEnd->setTimezone('UTC'));
 
         $r->fromRRule20('FREQ=YEARLY;INTERVAL=1;BYDAY=1TH;BYMONTH=3;COUNT=4');
         $this->assertEquals(Horde_Date_Recurrence::RECUR_YEARLY_WEEKDAY, $r->getRecurType());
@@ -1045,4 +1045,34 @@ class Horde_Date_RecurrenceTest extends PHPUnit_Framework_TestCase
         $this->assertEquals('America/New_York', $next->timezone);
     }
 
+    public function testBug12869RecurrenceEndFromIcalendar()
+    {
+        date_default_timezone_set('Europe/Amsterdam');
+        $iCal = new Horde_Icalendar();
+        $iCal->parsevCalendar(file_get_contents(__DIR__ . '/fixtures/bug12869.ics'));
+        $components = $iCal->getComponents();
+        foreach ($components as $content) {
+            if ($content instanceof Horde_Icalendar_Vevent) {
+                $start = new Horde_Date($content->getAttribute('DTSTART'));
+                $end = new Horde_Date($content->getAttribute('DTEND'));
+                $rrule = $content->getAttribute('RRULE');
+                $recurrence = new Horde_Date_Recurrence($start, $end);
+                $recurrence->fromRRule20($rrule);
+                break;
+            }
+        }
+
+        $after = array('year' => 2013, 'month' => 12);
+
+        $after['mday'] = 11;
+        $this->assertEquals('2013-12-12 13:45:00', (string)$recurrence->nextRecurrence($after));
+
+        $after['mday'] = 18;
+        $this->assertEquals('2013-12-19 13:45:00', (string)$recurrence->nextRecurrence($after));
+
+        $after['mday'] = 20;
+        $this->assertEquals('', (string)$recurrence->nextRecurrence($after));
+        date_default_timezone_set('Europe/Berlin');
+    }
+
 }
